/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : basethread.cpp                                                  */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 30/03/2006 */
/* email   : softinthebox@free.fr                      Rvision : 30/03/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "basethread.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

#include "basethread.h"
#include "myftpapp.h"
#include "myftpframe.h"
#include "myftp.h"

wxBaseThread::wxBaseThread(wxMyFTPFrame* frame, wxMyFTP* ftp) : wxThread()
{
   m_pFrame = frame;
   m_pFtp = ftp;
}

wxBaseThread::~wxBaseThread()
{
   // Accs exclusif au tableau de tches
   wxCriticalSectionLocker locker(wxGetApp().m_array_thread_critsect);

   wxArrayThread& threads = wxGetApp().m_threads;

   threads.Remove(this);   // Suppression dans le tableau

   if (threads.IsEmpty())  // Si le tableau est vide
      if (wxGetApp().m_waitingUntilAllDone)  // et que le flag de demande
      {  // de fin de toutes les tches est positionn alors
         wxGetApp().m_waitingUntilAllDone = false; // supprime le flag
         wxGetApp().m_semAllDone.Post();  // et envoi le signal
      }

   m_pFtp->m_ftp_critsect.Enter();  // Accs exclusif  l'objet wxFTP
   if (!m_pFtp->OnStreaming() && !wxGetApp().Used(m_pFtp))
   {  // L'objet wxFTP n'est pas en transfert ni utilis comme rpertoire distant
      // Accs exclusif au tableau d'objet wxFTP
      wxCriticalSectionLocker enter(wxGetApp().m_array_ftp_critsect);
      m_pFtp->SetDefaultTimeout(0);
      m_pFtp->Notify(false);  // Pour tre sur qu'aucune notification ne seras gnre
      wxGetApp().m_ftps.Remove(m_pFtp); // Supression de l'objet dans le tableau
      m_pFtp->m_ftp_critsect.Leave();   // Sortie du mode exclusif sur wxFTP
      m_pFtp->Destroy();   // Pour toutes les sockets Destroy est pfrable  delete
   }
   else
      m_pFtp->m_ftp_critsect.Leave();  // Sortie du mode exclusif sur wxFTP
}

void wxBaseThread::UpdateLog(const wxString& msg)
{
   if (!TestDestroy())     // Si aucune demande de destruction
   {
      wxMutexGuiEnter();   // Demande l'acs exclusif  l'interface

      m_pFrame->GetTextCtrlInfo()->AppendText(msg);      // Affiche le message
      m_pFrame->GetTextCtrlInfo()->AppendText(_T("\n")); // Ligne suivante

      wxMutexGuiLeave();   // Libre l'accs  l'inteface
   }
}

bool wxBaseThread::Connecting()
{
   // Instaure l'accs exclusif  l'objet wxFTP
   wxCriticalSectionLocker locker(m_pFtp->m_ftp_critsect);
   
   if (m_pFtp->IsDisconnected()) // si dj connect inutile de faire le traitement
   {
      int CountLoop = 0;

      UpdateLog(wxString::Format(_("Trying to connect to %s"),
                m_pFtp->Hostname().c_str()));

      m_pFtp->Connect();   // Connexion

      // Boucle en testant si demande de destruction ou connexion effective
      while (m_pFtp->IsDisconnected()&&!TestDestroy())
      {
         wxThread::Sleep(100);
         CountLoop++;

         if (CountLoop > 100)
            break;
      }
      if (TestDestroy())   // Si demande de destruction retour faux
      {
         m_pFtp->Close();
         return false;
      }
      if (m_pFtp->IsDisconnected()) // Si pas de connexion
      {  // Affichage d'un message et retour faux
         UpdateLog(wxString::Format(_("Unable to connect to %s"),
                   m_pFtp->Hostname().c_str()));
         return false;
      }
   }

   return true;
}
