/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : downloadthread.cpp                                              */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 04/04/2006 */
/* email   : softinthebox@free.fr                      Rvision : 04/04/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005                                  */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "downloadthread.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

#include <wx/wfstream.h>
#include <wx/filename.h>
#include "downloadthread.h"
#include "myftpapp.h"
#include "myftpframe.h"
#include "myftp.h"

void* wxDownloadThread::Entry()
{
   bool ret;
   size_t countread;
   wxFileName fileName;
   wxFileOutputStream* out = NULL;
   wxInputStream* in = NULL;
   
   UpdateTansfertItemStatus(umStarting);
      
   {
      wxCriticalSectionLocker locker(m_pFtp->m_ftp_critsect);
      // rcupration de la taille du fichier si possible
      int s = m_pFtp->GetFileSize(m_RemotePath, m_FileName);
      if (s != -1)   // Voir si la taille est retourne
         m_FileSize = (size_t)s;
   }
   // Cration du flux d'entre
   in = wxGetApp().GetInputStream(m_pFtp, m_RemotePath, m_FileName, m_AsciiMode);
   if (!in)
   {
      UpdateLog(wxString::Format(_("Couldn't get file %s from %s"),
                                 m_FileName.c_str(), m_pFtp->Hostname().c_str()));
   }
   else
   {
      // Cration du chemin global local
      fileName.AssignDir(m_LocalPath);
      fileName.SetFullName(m_FileName);
      // Cration du flux de sortie
      out = new wxFileOutputStream(fileName.GetFullPath());
      if (!out)
         UpdateLog(wxString::Format(_("Unable to create %s"),
                                    fileName.GetFullPath().c_str()));
   }

   {
      wxCriticalSectionLocker locker(m_pFtp->m_ftp_critsect);
      UpdateLog(m_pFtp->GetLastOpenStreamResult());
   }
   
   if (in && out)
   {
      UpdateTansfertItemStatus(umTransfering);   // Mise  jour interface

      while (!TestDestroy())
      {
         ret = !in->Read(m_Buffer, BUFFSIZE);   // Lire le flux
         countread = in->LastRead();   // Nombre d'octets lus
         if (countread)
         {  // Si quelque chose est lu, rire le buffer
            out->Write(m_Buffer, countread);
            m_SizeRead += countread;   // Incrmenter le compteur
            UpdateTransfertItem();     // Mettre  jour l'interface
         }

         if (ret) // Problme de lecture ou fin
            break;   // Sortir de la boucle
         wxThread::Sleep(100);
      }

      UpdateTansfertItemStatus(umFinishing);   // Mise  jour interface
      UpdateLog(wxString::Format(_("%s downloaded (%u bytes)"),
                                 m_FileName.c_str(), m_SizeRead));
   }
   
   // Supression des flux
   if (in)
      delete in;
   if (out)
      delete out;
   DeleteTransfertItem();  // Mettre  jour l'interface

   return NULL;
}
