/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : manageremote.cpp                                                */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 03/04/2006 */
/* email   : softinthebox@free.fr                      Rvision : 03/04/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "manageremote.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

#include <wx/dirctrl.h>
#include <wx/datetime.h>
#include "manageremote.h"
#include "myftpapp.h"
#include "myftpframe.h"
#include "myftp.h"

void* wxManageRemoteThread::Entry()
{
   wxJob* job;
   bool bRet, bAutoClose = true;

   wxGetApp().Using(m_pFtp);  // Activation du flag d'utilisation
   if (Connecting()) // Tentative de connexion
   {
      UpdateLog(wxString::Format(_("Connected to %s"), m_pFtp->Hostname().c_str()));
      SetConnect(true);
   }
   else
   {
      wxGetApp().Using(0); // Dsactivation du flag d'utilisation
      return NULL;
   }
   UpdateMenu();
   UpdateDisplay();
   UpdateLog(m_pFtp->GetLastResult());
   UpdatePath();

   // On boucle tant que la connexion est active et qu'aucune demande de destruction
   while (m_pFtp->IsConnected()&&!TestDestroy())
   {
      job = GetNextJob();  // Rcupration de travail

      if ((job == NULL)||(job->jobsFtp == mfjNone))
         wxThread::Sleep(100);   // Aucun on attend un peu 1/10e de seconde
      else
         switch (job->jobsFtp)   // Dispatche le travail
         {
            case mfjNone:  // pour viter un warning
               break;
            case mfjRefresh :
               UpdateDisplay();
               UpdatePath();
               break;
            case mfjChangeDir :
               if (job->jobsString1 == _T(".."))   // Si demande rpertoire parent
                  bRet = m_pFtp->CdUp();  // Utilisation de la nouvelle commande
               else
                  bRet = m_pFtp->ChDir(job->jobsString1);
               if (bRet)
               {
                  UpdateDisplay();
                  UpdatePath();
               }
               break;
            case mfjCreateDir :
               bRet = m_pFtp->MkDir(job->jobsString1);
               UpdateLog(m_pFtp->GetLastResult());
               if (bRet)
               {
                  UpdateDisplay();
                  UpdatePath();
               }
               break;
            case mfjRemoveDir :
               m_pFtp->RmDir(job->jobsString1);
               UpdateLog(m_pFtp->GetLastResult());
               break;
            case mfjDelete :
               m_pFtp->RmFile(job->jobsString1);
               UpdateLog(m_pFtp->GetLastResult());
               break;
            case mfjRename :
               bRet = m_pFtp->Rename(job->jobsString1, job->jobsString2);
               UpdateLog(m_pFtp->GetLastResult());
               if (bRet)
               {
                  UpdateDisplay();
                  UpdatePath();
               }
               break;
            case mfjDisconnect :    // demande de dconnexion de l'utilisateur
               ClearJobs();         // Effacement de la file de travaux
               delete job;          // Suppression du travail
               bAutoClose = false;
               SetConnect(false);
               UpdateMenu();
               ClearDisplay();      // Effacement de la liste
               UpdateLog(wxString::Format(_("Disconnected from %s"), m_pFtp->Hostname().c_str()));
               wxGetApp().Using(0); // Dsactivation du flag d'utilisation
               return NULL;         // Sortie de la tche
         }
      if (job != NULL)
         delete job; // Suppression du travail
   }
   if (bAutoClose&&!TestDestroy())
   {  // Fermeture automatique
      SetConnect(false);
      UpdateMenu();
      ClearDisplay();
      UpdateLog(wxString::Format(_("Disconnected from %s (timeout)"), m_pFtp->Hostname().c_str()));
   }
   ClearJobs();         // Effacement de la file de travaux
   wxGetApp().Using(0); // Dsactivation du flag d'utilisation
   return NULL;
}

void wxManageRemoteThread::UpdateDisplay()
{
   wxArrayString files, fTmp;
   wxListCtrl* remote;
   long m_count;
   int image_id;
   bool bRepParent = false;
   bool IsCnx;

   wxMutexGuiEnter();   // Utilisation exclusive de la GUI

   remote = (wxListCtrl*)m_pFrame->GetRemoteFiles();
   remote->DeleteAllItems();  // Effacement de la liste

   wxMutexGuiLeave();   // Fin de l'utilisation exclusive

   m_pFtp->m_ftp_critsect.Enter();
   IsCnx = m_pFtp->IsConnected();   // Vrification de la connexion
   m_pFtp->m_ftp_critsect.Leave();
   if (IsCnx)
   {
      wxString dirItems, tmp, name, type, size, date, attr;
      size_t count;
      bool bRet;

      m_pFtp->m_ftp_critsect.Enter();
      bRet = m_pFtp->GetDirList(files);   // rcupration de la liste de fichiers
      m_pFtp->m_ftp_critsect.Leave();

      if (!bRet)
         UpdateLog(_("Problem to receive files list!"));
      else
      {  
         count = files.GetCount();
         for (size_t i = 0; i < count; i++)
         {  
            dirItems = files.Item(i);
            // Exemple de chaines reues
            // drwxr-xr-x 1 ftp ftp              0 Sep 05 19:27 bin
            // -r--r--r--   1 guilhem  lavaux      12738 Jan 16 20:17 cmndata.cpp
            // Dcoupage de chaque chaine
            attr = dirItems.BeforeFirst(' ');
            tmp  = dirItems.AfterFirst(' ').Strip(wxString::leading);
            type = tmp.BeforeFirst(' ');
            tmp  = tmp.AfterFirst(' ').Strip(wxString::leading);

            tmp  = tmp.AfterFirst(' ').Strip(wxString::leading);
            tmp  = tmp.AfterFirst(' ').Strip(wxString::leading);

            size = tmp.BeforeFirst(' ');
            tmp  = tmp.AfterFirst(' ').Strip(wxString::leading);
            date = tmp.Left(12);
            name = tmp.Mid(13);
            fTmp.Add(name);
            if (name != _T(".")) // Exclure le lien vers le rpertoire lui mme
            {
               if (name == _T(".."))   // On vrifie l'existance d'un retour
                  bRepParent = true;   // au rpertoire parent
               wxMutexGuiEnter();      // Utilisation exclusive de la GUI
               m_count = remote->GetItemCount();
               m_count = remote->InsertItem(m_count, name); // Insertion de l'lment
               if (attr.substr(0, 1) == _T("d"))   // est-ce un rpertoire?
               {  // on indique que c'est un rpertoire au lieu de la taille
                  remote->SetItem(m_count, 1, _T("<DIR>"));
                  // Utilisation de wxFileIconsTable::folder pour l'image de rpertoire
                  remote->SetItemImage(m_count, wxFileIconsTable::folder);
                  // On indique que c'est un rpertoire en positionnant un index ngatif
                  remote->SetItemData(m_count, -(i + 1));
               }
               else
               {
                  remote->SetItem(m_count, 1, size);  // On indique la taille
                  image_id = wxFileIconsTable::file;  // Image par dfaut pour un fichier
                  if (name.Find(_T('.')) != wxNOT_FOUND) // Si possible on cherche l'icne correspondante
                     image_id = wxTheFileIconsTable->GetIconID(name.AfterLast(_T('.')));
                  remote->SetItemImage(m_count, image_id);
                  // on indique que c'est un fichier en positionnant un index positif
                  remote->SetItemData(m_count, i + 1);
               }
               wxDateTime dateTime;
               dateTime.ParseDate(date.c_str());
               tmp = date.AfterLast(' ');
               date = dateTime.FormatISODate() + _T(" ");
               if (tmp.Find(':') != -1)
                  date += tmp;
               else  // Si l'heure n'est pas indique mettre 12:00 par dfaut
                  date += _T("12:00");
               remote->SetItem(m_count, 2, date);  // On indique la date et l'heure
               wxMutexGuiLeave();      // Fin de l'utilisation exclusive
            }
         }
         if (bRepParent == false)
         {  // Il n'y a pas de retour au parent on l'ajoute
            wxMutexGuiEnter();
            name = _T("..");
            fTmp.Add(name);
            count = remote->GetItemCount();
            m_count = remote->InsertItem(count, name);
            remote->SetItem(m_count, 1, _T("<DIR>"));
            remote->SetItemImage(m_count, wxFileIconsTable::folder);
            remote->SetItemData(m_count, -(count + 1));
            wxDateTime dateTime = wxDateTime::Now();
            date = dateTime.FormatISODate() + _T(" ") + dateTime.FormatISOTime().Left(5);
            remote->SetItem(m_count, 2, date);
            wxMutexGuiLeave();
         }
         wxMutexGuiEnter();
         // Tri par ordre alpha avec les rpertoire d'abord
         // Pour cel on se sert de la liste de nom que l'on fourni en paramtre
         // et des index positionns par SetItemData ngatif pour les rpertoires
         // et positif pour les fichiers
         remote->SortItems(CompareList, (long)&fTmp);
         wxMutexGuiLeave();
      }
   }
}

void wxManageRemoteThread::ClearDisplay()
{
   wxListCtrl* remote;

   wxMutexGuiEnter();

   remote = (wxListCtrl*)m_pFrame->GetRemoteFiles();
   remote->DeleteAllItems();

   wxMutexGuiLeave();
}

void wxManageRemoteThread::SetConnect(bool value)
{
   wxCriticalSectionLocker locker(wxGetApp().m_data_critsect);
   wxGetApp().bConnected = value;
}

void wxManageRemoteThread::UpdateMenu()
{
   wxMutexGuiEnter();
   m_pFrame->EnableMenu();
   wxMutexGuiLeave();
}

void wxManageRemoteThread::UpdatePath()
{
   wxString path = m_pFtp->Pwd();
   wxCriticalSectionLocker locker(wxGetApp().m_data_critsect);

   wxGetApp().remotePath = path;
}

wxJob* wxManageRemoteThread::GetNextJob()
{
   wxJob* job = NULL;

   wxCriticalSectionLocker locker(wxGetApp().m_array_job_critsect);

   if (wxGetApp().m_jobs.GetCount())
   {
      job = wxGetApp().m_jobs[0];
      wxGetApp().m_jobs.Remove(job);
   }
   return job;
}

void wxManageRemoteThread::ClearJobs()
{
   wxJob* job;
   wxCriticalSectionLocker locker(wxGetApp().m_array_job_critsect);

   size_t count = wxGetApp().m_jobs.GetCount();

   if (count)
   {
      while (!wxGetApp().m_jobs.IsEmpty())
      {
         job = wxGetApp().m_jobs.Last();
         wxGetApp().m_jobs.Remove(job);
         delete job;
      }
   }
}
