/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : myftpapp.cpp                                                    */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 27/02/2006 */
/* email   : softinthebox@free.fr                      Rvision : 03/03/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/
#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "myftpapp.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include <wx/intl.h>
#include "myftpapp.h"
#include "myftpframe.h"
#include "myftp.h"

IMPLEMENT_APP(wxMyFTPApp)

wxMyFTPApp::wxMyFTPApp() : m_semAllDone()
{
   m_UsedFTP = 0;
   m_waitingUntilAllDone = false;
   bConnected = false;
}

bool wxMyFTPApp::OnInit()
{
   wxApp::OnInit();

   // Gestion de la langue
   // Ajout des prfixes possible de chemin d'accs aux catalogues
   wxLocale::AddCatalogLookupPathPrefix(wxT("."));
   wxLocale::AddCatalogLookupPathPrefix(wxT(".."));
   wxLocale::AddCatalogLookupPathPrefix(_T("locale"));
   // Mise en place de la langue par dfaut du systme
   m_locale.Init(wxLANGUAGE_DEFAULT);
   {
      wxLogNull noLog;  // Supprime les erreurs si les catalogues n'existent pas
      // Catalogue de l'application
      m_locale.AddCatalog(_T("myftp"));
      // Catalogue de wxWidgets
      m_locale.AddCatalog(_T("wxstd"));
   }

   wxMyFTPFrame* mainWindow = new wxMyFTPFrame(NULL);
   mainWindow->SetSize(wxSize(640, 480));
   mainWindow->Show(true);

   return true;
}

wxString wxMyFTPApp::GetRemotePath()
{
   wxCriticalSectionLocker locker(m_data_critsect);
   return remotePath;
}

bool wxMyFTPApp::GetConnected()
{
   wxCriticalSectionLocker locker(m_data_critsect);
   return bConnected;
}

wxMyFTP* wxMyFTPApp::GetFTP(const wxString& hostname, unsigned short service, const wxString& user)
{
   wxMyFTP* ftp = 0;

   size_t count = m_ftps.GetCount();

   for (size_t i = 0; i < count; i++)
   {  // On boucle dans le tableau tant qu'une instance avec les proprits
      // voulues n'est pas trouv
      if ((m_ftps[i]->Service() == service)&&
          (m_ftps[i]->Hostname() == hostname)&&
          (m_ftps[i]->User() == user))
      {
         ftp = m_ftps[i];
         break;
      }
   }

   return ftp;
}

wxMyFTP* wxMyFTPApp::CreateFTP(const wxString& hostname, unsigned short service,
                   const wxString& user, const wxString& password, bool passive)
{
   // Une instance avec les bons paramtres existe t'elle?
   wxMyFTP* ftp = GetFTP(hostname, service, user);
   if (!ftp)
   {  // Non alors on l'alloue et on l'ajoute au tableau
      ftp = new wxMyFTP(hostname, service);
      if (user != wxEmptyString)
      {
         ftp->SetUser(user);
         ftp->SetPassword(password);
      }
      ftp->SetDefaultTimeout(10);
      ftp->SetPassive(passive);
      m_ftps.Add(ftp);
   }
   return ftp;
}

wxOutputStream* wxMyFTPApp::GetOutputStream(wxMyFTP* ftp, const wxString& path,
                                            const wxString& filename, bool ascii)
{
   wxCriticalSectionLocker enter(ftp->m_ftp_critsect);

   if (ftp)
      return ftp->GetOutputStream(path, filename, ascii);
   else
      return NULL;
}

wxInputStream* wxMyFTPApp::GetInputStream(wxMyFTP* ftp, const wxString& path,
                                          const wxString& filename, bool ascii)
{
   wxCriticalSectionLocker enter(ftp->m_ftp_critsect);

   if (ftp)
      return ftp->GetInputStream(path, filename, ascii);
   else
      return NULL;
}

void wxMyFTPApp::Using(wxMyFTP* ftp)
{
   wxCriticalSectionLocker enter(m_used_critsect);

   m_UsedFTP = ftp;
}

bool wxMyFTPApp::Used(wxMyFTP* ftp)
{
   wxCriticalSectionLocker enter(m_used_critsect);

   return (m_UsedFTP == ftp);
}

bool wxMyFTPApp::CurrentOnStreaming()
{
   wxCriticalSectionLocker enter(m_used_critsect);

   if (m_UsedFTP)
   {
      wxCriticalSectionLocker locker(m_UsedFTP->m_ftp_critsect);

      return m_UsedFTP->OnStreaming();
   }
   else
      return false;
}

int wxCALLBACK CompareList(long item1, long item2, long sortData)
{
   wxArrayString* files;

   files = (wxArrayString*)sortData;
   if ((item1 < 0)&&(item2 > 0))
      return -1;
   else if ((item1 > 0)&&(item2 < 0))
      return 1;
   else
      return (*files)[abs(item1) - 1].CmpNoCase((*files)[abs(item2) - 1]);
}
