/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : myftpdnd.cpp                                                    */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 07/04/2006 */
/* email   : softinthebox@free.fr                      Rvision : 11/04/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "myftpdnd.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

#include "myftpdnd.h"
#include "myftpframe.h"

static const wxChar* UploadFormatId = _T("wxDndUploadFileDataObject");
static const wxChar* DownloadFormatId = _T("wxDndDownloadFileDataObject");

wxDragResult wxDndRemoteTarget::OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
{
   if (!wxGetApp().GetConnected())  // Si aucune connection active
      return wxDragNone;            // on ne peut pas dposer
   // Pas de  dplacements uniquement des copies
   return (def == wxDragMove ? wxDragCopy : def);  
}

wxDragResult wxDndRemoteTarget::OnData(wxCoord x, wxCoord y, wxDragResult def)
{
   if (!GetData())   // rcupration des donnes
      return wxDragNone;

   wxDndUploadFile* TransFile;
   // Rcupration d'un pointeur sur l'objet donnes (wxDndUploadFile)
   TransFile = ((wxDndUploadFileDataObject *)GetDataObject())->GetTransFile();
   size_t nFiles = TransFile->GetCount();
   // Appel de la fonction Upload de la fentre principale pour
   // chaque nom de fichier dans l'instance d'objet wxDndUploadFile
   for (size_t i = 0; i < nFiles; i++) 
      m_Frame->Upload(TransFile->m_Path, TransFile->Item(i));

   if (nFiles)
   {  // Si des fichiers tait prsent envoie d'un message pour lancer la tche
      wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, ID_CMD_LAUNCHTHREAD);
      wxPostEvent(m_Frame, event);
   }
   
   return wxDragCopy;
}

wxDndUploadFile::wxDndUploadFile(const wxDndUploadFile& dndtransfile)
{
   m_Path = dndtransfile.m_Path;
   m_Files = dndtransfile.m_Files;
}

wxDndDownloadFile::wxDndDownloadFile(const wxDndDownloadFile& dndtransfile)
{
   m_Path = dndtransfile.m_Path;
   m_Hostname = dndtransfile.m_Hostname;
   m_Service = dndtransfile.m_Service;
   m_User = dndtransfile.m_User;
   m_Password = dndtransfile.m_Password;
   m_Mode = dndtransfile.m_Mode;
   m_Files = dndtransfile.m_Files;
}

wxDndUploadFileDataObject::wxDndUploadFileDataObject(wxDndUploadFile* dndtransfile)
{
   if (dndtransfile != 0)  // Si aucun object donne n'est fourni en crer un
      m_DndTransFile = new wxDndUploadFile(*dndtransfile);
   else
      m_DndTransFile = 0;
      
   wxDataFormat transfileformat; // Positionner l'identifiant du format
   transfileformat.SetId(UploadFormatId);
   SetFormat(transfileformat);
}

size_t wxDndUploadFileDataObject::GetDataSize() const
{
   size_t ret, count;
   
   if (m_DndTransFile)  // Vrification de l'existance d'un objet donne
   {  // Chemin et noms de fichiers spars par \n
      ret = m_DndTransFile->m_Path.Len() + 1; // On ajoute 1 caractre pour \n
      count = m_DndTransFile->GetCount();
      ret += count;  // On ajoute count caractre \n
      for (size_t i = 0; i < count; i++)
         ret += m_DndTransFile->Item(i).Len();
   }
   else
      ret = 0;
   return ret;
}

bool wxDndUploadFileDataObject::GetDataHere(void* buf) const
{
   if (m_DndTransFile)  // Vrification de l'existance d'un objet donne
   {
      wxString tmp;
      size_t count;
      // remplissage du tampon en sparant par des \n
      tmp = m_DndTransFile->m_Path + _T("\n");  // le premier est le chemin
      count = m_DndTransFile->GetCount();
      for (size_t i = 0; i < count; i++)  // les autres sont les noms de fichiers
         tmp += m_DndTransFile->Item(i) + _T("\n");
      memcpy(buf, tmp.c_str(), tmp.Len());
      return true;
   }
   return false;
}

bool wxDndUploadFileDataObject::SetData(size_t len, const void* buf)
{
   if (len != 0)
   {
      wxString tmp1, tmp2;
      
      if (!m_DndTransFile) // Crer un objet donne si inexistant
         m_DndTransFile = new wxDndUploadFile;
      tmp1 = wxString((const char*)buf, len);
      tmp2 = tmp1.BeforeFirst(_T('\n'));  // dcoupage du tampon  chaque \n
      tmp1 = tmp1.AfterFirst(_T('\n'));   
      m_DndTransFile->m_Path = tmp2;      // Le premier est le chemin
      m_DndTransFile->m_Files.Clear();
      while (!tmp1.IsEmpty())
      {
         tmp2 = tmp1.BeforeFirst(_T('\n'));  // dcoupage du tampon  chaque \n
         tmp1 = tmp1.AfterFirst(_T('\n'));
         if (!tmp2.IsEmpty())
            m_DndTransFile->AddFile(tmp2);   // Les autres sont les noms de fichiers
      }
   }

   return true;
}

wxDndDownloadFileDataObject::wxDndDownloadFileDataObject(wxDndDownloadFile* dndtransfile)
{
   if (dndtransfile != 0) // Si aucun object donne n'est fourni en crer un
      m_DndTransFile = new wxDndDownloadFile(*dndtransfile);
   else
      m_DndTransFile = 0;

   wxDataFormat transfileformat; // Positionner l'identifiant du format
   transfileformat.SetId(DownloadFormatId);
   SetFormat(transfileformat);
}

size_t wxDndDownloadFileDataObject::GetDataSize() const
{
   size_t ret, count;
   wxString tmp;
   
   if (m_DndTransFile)
   {
      ret = 1; // pour m_Mode (Binary, ascIi, Auto)
      tmp = wxString::Format(_T("%u") , (unsigned int)m_DndTransFile->m_Service);
      ret += tmp.Len() + 1; // On ajoute 1 caractre pour \n
      ret += m_DndTransFile->m_Path.Len() + 1; // On ajoute 1 caractre pour \n
      ret += m_DndTransFile->m_Hostname.Len() + 1; // On ajoute 1 caractre pour \n
      ret += m_DndTransFile->m_User.Len() + 1; // On ajoute 1 caractre pour \n
      ret += m_DndTransFile->m_Password.Len() + 1; // On ajoute 1 caractre pour \n

      count = m_DndTransFile->GetCount();
      ret += count;  // On ajoute count caractre \n
      for (size_t i = 0; i < count; i++)
         ret += m_DndTransFile->Item(i).Len();
   }
   else
      ret = 0;
   return ret;
}

bool wxDndDownloadFileDataObject::GetDataHere(void* buf) const
{
   if (m_DndTransFile)
   {
      wxString tmp;
      size_t count;
      // Un caractre pour le mode de transfert
      switch (m_DndTransFile->m_Mode)
      {
         case TM_ASCII  : tmp = _T("I");
            break;
         case TM_BINARY : tmp = _T("B");
            break;
         case TM_AUTO   : tmp = _T("A");
            break;
         default : return false;
      }
      // Puis le reste des lments sont spars pas \n
      tmp += wxString::Format(_T("%u\n") , (unsigned int)m_DndTransFile->m_Service);
      tmp += m_DndTransFile->m_Path + _T("\n");
      tmp += m_DndTransFile->m_Hostname + _T("\n");
      tmp += m_DndTransFile->m_User + _T("\n");
      tmp += m_DndTransFile->m_Password + _T("\n");
      // On ajoute les noms de fichiers spars \n
      count = m_DndTransFile->GetCount();
      for (size_t i = 0; i < count; i++)
         tmp += m_DndTransFile->Item(i) + _T("\n");
      memcpy(buf, tmp.c_str(), tmp.Len());
      return true;
   }
   return false;
}

bool wxDndDownloadFileDataObject::SetData(size_t len, const void* buf)
{
   if (len != 0)
   {
      wxString tmp1, tmp2, srv;
      unsigned long usrv;

      if (!m_DndTransFile)
         m_DndTransFile = new wxDndDownloadFile;
      tmp1 = wxString((const char*)buf, len);

      switch (tmp1.GetChar(0))
      {
         case _T('I') : m_DndTransFile->m_Mode = TM_ASCII;
            break;
         case _T('B') : m_DndTransFile->m_Mode = TM_BINARY;
            break;
         case _T('A') : m_DndTransFile->m_Mode = TM_AUTO;
            break;
         default : return false;
      }
      tmp2 = tmp1.BeforeFirst(_T('\n')).Mid(1);
      tmp1 = tmp1.AfterFirst(_T('\n'));
      srv = tmp2;
      if (!srv.ToULong(&usrv))
         return false;
      m_DndTransFile->m_Service = (unsigned short)usrv;
      tmp2 = tmp1.BeforeFirst(_T('\n'));
      tmp1 = tmp1.AfterFirst(_T('\n'));
      m_DndTransFile->m_Path = tmp2;
      tmp2 = tmp1.BeforeFirst(_T('\n'));
      tmp1 = tmp1.AfterFirst(_T('\n'));
      m_DndTransFile->m_Hostname = tmp2;
      tmp2 = tmp1.BeforeFirst(_T('\n'));
      tmp1 = tmp1.AfterFirst(_T('\n'));
      m_DndTransFile->m_User = tmp2;
      tmp2 = tmp1.BeforeFirst(_T('\n'));
      tmp1 = tmp1.AfterFirst(_T('\n'));
      m_DndTransFile->m_Password = tmp2;

      m_DndTransFile->m_Files.Clear();
      while (!tmp1.IsEmpty())
      {
         tmp2 = tmp1.BeforeFirst(_T('\n'));
         tmp1 = tmp1.AfterFirst(_T('\n'));
         if (!tmp2.IsEmpty())
            m_DndTransFile->AddFile(tmp2);
      }
      return true;
   }
   return false;
}

wxDragResult wxDndLocaleTarget::OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
{  // Pas de  dplacements uniquement des copies
   return (def == wxDragMove ? wxDragCopy : def);
}

wxDragResult wxDndLocaleTarget::OnData(wxCoord x, wxCoord y, wxDragResult def)
{
   if (!GetData())   // rcupration des donnes
      return wxDragNone;

   wxDndDownloadFile* TransFile;
   // Rcupration d'un pointeur sur l'objet donnes (wxDndDownloadFile)
   TransFile = ((wxDndDownloadFileDataObject *)GetDataObject())->GetTransFile();
   // Appel de la fonction Download de la fentre principale pour
   // chaque nom de fichier dans l'instance d'objet wxDndDownloadFile
   size_t nFiles = TransFile->GetCount();
   for (size_t i = 0; i < nFiles; i++)
      m_Frame->Download(TransFile->m_Path, TransFile->Item(i),
                        TransFile->m_Hostname, TransFile->m_Service,
                        TransFile->m_User, TransFile->m_Password,
                        TransFile->m_Mode);

   if (nFiles)
   {  // Si des fichiers tait prsent envoie d'un message pour lancer la tche
      wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, ID_CMD_LAUNCHTHREAD);
      wxPostEvent(m_Frame, event);
   }
   
   return wxDragCopy;
}
