/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : myftpdnd.h                                                      */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 07/04/2006 */
/* email   : softinthebox@free.fr                      Rvision : 07/04/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2).

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/

#ifndef _DND_H_
#define _DND_H_

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "myftpdnd.cpp"
#endif

#include <wx/dnd.h>
#include <wx/listctrl.h>
class wxMyFTPFrame;

class wxDndUploadFile
{
   public:

      wxDndUploadFile() {}
      wxDndUploadFile(const wxString& path) : m_Path(path) {}
      wxDndUploadFile(const wxDndUploadFile& dndtransfile);
      ~wxDndUploadFile() {m_Files.Clear();}
      
      void AddFile(const wxString& file)
            {if (!file.IsEmpty()) m_Files.Add(file);}
      
      wxString& Item(size_t nIndex) const {return m_Files.Item(nIndex);}
      size_t GetCount() const {return m_Files.GetCount();}
      
      wxString m_Path;
      wxArrayString m_Files;
};

class wxDndUploadFileDataObject : public wxDataObjectSimple
{
   public:

      wxDndUploadFileDataObject(wxDndUploadFile* dndtransfile = 0);
      virtual ~wxDndUploadFileDataObject() {delete m_DndTransFile;}
      
      virtual size_t GetDataSize() const;
      virtual bool GetDataHere(void* buf) const;
      virtual bool SetData(size_t len, const void* buf);

      wxDndUploadFile* GetTransFile() {return m_DndTransFile;}
      
   private:

      wxDndUploadFile* m_DndTransFile;
};

class wxDndRemoteTarget : public wxDropTarget
{
   public:

      wxDndRemoteTarget(wxListCtrl* owner, wxMyFTPFrame* frame) :
         wxDropTarget(new wxDndUploadFileDataObject)
         {
            m_Owner = owner;
            m_Frame = frame;
         }


      virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def);
      virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def);

   private:

      wxListCtrl* m_Owner;
      wxMyFTPFrame* m_Frame;
};

class wxDndDownloadFile
{
   public:

      wxDndDownloadFile() : m_Service(0) {}
      wxDndDownloadFile(const wxString& path, const wxString& hostname,
                        unsigned short service, const wxString& user,
                        const wxString& password, int mode) :
         m_Path(path), m_Hostname(hostname), m_Service(service), m_User(user),
         m_Password(password), m_Mode(mode) {}

      wxDndDownloadFile(const wxDndDownloadFile& dndtransfile);
      ~wxDndDownloadFile() {m_Files.Clear();}

      void AddFile(const wxString& file)
            {if (!file.IsEmpty()) m_Files.Add(file);}

      wxString& Item(size_t nIndex) const {return m_Files.Item(nIndex);}
      size_t GetCount() const {return m_Files.GetCount();}

      wxString m_Path;
      wxString m_Hostname;
      unsigned short m_Service;
      wxString m_User;
      wxString m_Password;
      int m_Mode;
      wxArrayString m_Files;
};

class wxDndDownloadFileDataObject : public wxDataObjectSimple
{
   public:

      wxDndDownloadFileDataObject(wxDndDownloadFile* dndtransfile = 0);
      virtual ~wxDndDownloadFileDataObject() {delete m_DndTransFile;}

      virtual size_t GetDataSize() const;
      virtual bool GetDataHere(void* buf) const;
      virtual bool SetData(size_t len, const void* buf);

      wxDndDownloadFile* GetTransFile() {return m_DndTransFile;}

   private:

      wxDndDownloadFile* m_DndTransFile;
};

class wxDndLocaleTarget : public wxDropTarget
{
   public:

      wxDndLocaleTarget(wxListCtrl* owner, wxMyFTPFrame* frame) :
         wxDropTarget(new wxDndDownloadFileDataObject)
         {
            m_Owner = owner;
            m_Frame = frame;
         }


      virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def);
      virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def);

   private:

      wxListCtrl* m_Owner;
      wxMyFTPFrame* m_Frame;
};

#endif   // _DND_H_
