/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : myftpframe.cpp                                                  */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 28/02/2006 */
/* email   : softinthebox@free.fr                      Rvision : 05/03/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/
#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "myftpframe.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include <sys/stat.h>
#include <dirent.h>
#include <wx/datetime.h>
#include <wx/dirctrl.h>
#include <wx/filename.h>
#include "myftpframe.h"
#include "myftpapp.h"
#include "cnxdlg.h"
#include "myftp.h"
#include "manageremote.h"
#include "downloadthread.h"
#include "uploadthread.h"

#include "myftp_logo.xpm"
#include "tool_ascii.xpm"
#include "tool_auto.xpm"
#include "tool_binary.xpm"
#include "tool_connecting.xpm"
#include "tool_disconnecting.xpm"
#include "tool_download.xpm"
#include "tool_upload.xpm"

// Control Id
#define ID_TOOLBAR            10101
#define ID_STATUSBAR          10102
#define ID_INFORMATION        10103
#define ID_SPLITTERWINDOW     10104
#define ID_FILES_PANEL        10105
#define ID_LOCAL_FILES        10106
#define ID_REMOTE_FILES       10107
#define ID_TRANSFERTS_LIST    10108

// Menu Item Id
#define ID_MNU_CONNECTING     10201
#define ID_MNU_DISCONNECTING  10202
#define ID_MNU_QUIT           10203

#define ID_MNU_UPLOAD         10301
#define ID_MNU_CHANGE_LOCAL   10302
#define ID_MNU_CREATE_LOCAL   10303
#define ID_MNU_DEL_LOCAL      10304
#define ID_MNU_REFRESH_LOCAL  10305

#define ID_MNU_DOWNLOAD       10401
#define ID_MNU_CREATE_REMOTE  10402
#define ID_MNU_DEL_REMOTE     10403
#define ID_MNU_RENAME_REMOTE  10404
#define ID_MNU_REFRESH_REMOTE 10405

#define ID_MNU_ABOUT          10501

#define ID_MNU_BINARY         10601
#define ID_MNU_ASCII          10602
#define ID_MNU_AUTO           10603

BEGIN_EVENT_TABLE(wxMyFTPFrame, wxFrame)
   EVT_MENU(ID_MNU_CONNECTING, wxMyFTPFrame::OnMnuConnectingClick)
   EVT_MENU(ID_MNU_DISCONNECTING, wxMyFTPFrame::OnMnuDisconnectingClick)
   EVT_MENU(ID_MNU_QUIT, wxMyFTPFrame::OnMnuQuitClick)
   EVT_MENU(ID_MNU_UPLOAD, wxMyFTPFrame::OnMnuUploadClick)
   EVT_MENU(ID_MNU_CHANGE_LOCAL, wxMyFTPFrame::OnMnuChangeLocalClick)
   EVT_MENU(ID_MNU_CREATE_LOCAL, wxMyFTPFrame::OnMnuCreateLocalClick)
   EVT_MENU(ID_MNU_DEL_LOCAL, wxMyFTPFrame::OnMnuDelLocalClick)
   EVT_MENU(ID_MNU_REFRESH_LOCAL, wxMyFTPFrame::OnMnuRefreshLocalClick)
   EVT_MENU(ID_MNU_DOWNLOAD, wxMyFTPFrame::OnMnuDownloadClick)
   EVT_MENU(ID_MNU_DEL_REMOTE, wxMyFTPFrame::OnMnuDelRemoteClick)
   EVT_MENU(ID_MNU_RENAME_REMOTE, wxMyFTPFrame::OnMnuRenameClick)
   EVT_MENU(ID_MNU_CREATE_REMOTE, wxMyFTPFrame::OnMnuCreateRemoteClick)
   EVT_MENU(ID_MNU_REFRESH_REMOTE, wxMyFTPFrame::OnMnuRefreshRemoteClick)
   EVT_MENU(ID_MNU_ABOUT, wxMyFTPFrame::OnMnuAboutClick)
   EVT_MENU(ID_MNU_BINARY, wxMyFTPFrame::OnMnuTransfertModeClick)
   EVT_MENU(ID_MNU_ASCII, wxMyFTPFrame::OnMnuTransfertModeClick)
   EVT_MENU(ID_MNU_AUTO, wxMyFTPFrame::OnMnuTransfertModeClick)

   EVT_MENU(ID_CMD_LAUNCHTHREAD, wxMyFTPFrame::OnEvtLaunchThread)
   
   EVT_LIST_ITEM_SELECTED(ID_LOCAL_FILES, wxMyFTPFrame::OnFilesChangeSelection)
   EVT_LIST_ITEM_DESELECTED(ID_LOCAL_FILES, wxMyFTPFrame::OnFilesChangeSelection)
   EVT_LIST_BEGIN_DRAG(ID_LOCAL_FILES, wxMyFTPFrame::OnLocalFilesBeginDrag)
   EVT_LIST_ITEM_ACTIVATED(ID_LOCAL_FILES, wxMyFTPFrame::OnLocalFilesItemActivated)

   EVT_LIST_ITEM_SELECTED(ID_REMOTE_FILES, wxMyFTPFrame::OnFilesChangeSelection)
   EVT_LIST_ITEM_DESELECTED(ID_REMOTE_FILES, wxMyFTPFrame::OnFilesChangeSelection)
   EVT_LIST_BEGIN_DRAG(ID_REMOTE_FILES, wxMyFTPFrame::OnRemoteFilesBeginDrag)
   EVT_LIST_ITEM_ACTIVATED(ID_REMOTE_FILES, wxMyFTPFrame::OnRemoteFilesItemActivated)
END_EVENT_TABLE()

wxMyFTPFrame::wxMyFTPFrame(wxWindow* parent, wxWindowID id, const wxString& caption,
                           const wxPoint& pos, const wxSize& size, long style)
{
   Create(parent, id, caption, pos, size, style);
}

wxMyFTPFrame::~wxMyFTPFrame()
{
   wxThread *thread;
   wxTransJob* transjob;
   size_t count, i;
   // demande d'accs exclusif au tableau de tches
   wxGetApp().m_array_thread_critsect.Enter();

   const wxArrayThread& threads = wxGetApp().m_threads;
   count = threads.GetCount();

   if (count)
   {
      wxGetApp().m_waitingUntilAllDone = true;

      while (!threads.IsEmpty()) // Tant que le tableau n'est pas vide
      {
         thread = threads.Last();   // Rcuprer un pointeur sur la dernire tche
         wxGetApp().m_array_thread_critsect.Leave();  // Librer le tableau de tche
         thread->Delete(); // demande de destruction de la tche
         wxGetApp().m_array_thread_critsect.Enter();  // Reprise du tableau de tche
      }
   }

   wxGetApp().m_array_thread_critsect.Leave();  // Libration du tableau de tche

   if (count)
      wxGetApp().m_semAllDone.Wait();  // Attendre que le semaphore soit positif

   // Dstruction de toutes les instances restantes de wxTransJob dans la liste
   count = m_TransfertList->GetItemCount();
   for (i = 0; i < count; i++)
   {
      transjob = (wxTransJob*)m_TransfertList->GetItemData(i);
      if (transjob != 0)
         delete transjob;
   }
}

bool wxMyFTPFrame::Create(wxWindow* parent, wxWindowID id, const wxString& caption,
                        const wxPoint& pos, const wxSize& size, long style)
{
   m_TransfertMode = TM_AUTO;
   m_Service = 21;
   m_Passive = false;

   m_MnuFile = NULL;
   m_MnuLocal = NULL;
   m_MnuRemote = NULL;
   
   m_TextCtrlInfo = NULL;
   m_HorzSplitter = NULL;
   m_LocalFiles = NULL;
   m_RemoteFiles = NULL;
   m_TransfertList = NULL;

   wxFrame::Create(parent, id, caption, pos, size, style);
   CreateMenu();
   CreateToolbar();
   CreateControls();
   GetSizer()->Fit(this);
   GetSizer()->SetSizeHints(this);

   m_HorzSplitter->SetSashPosition(65);

   EnableMenu();
   SetIcon(wxIcon(myftp_logo_xpm));
   return TRUE;
}

void wxMyFTPFrame::CreateMenu()
{
   wxMenuBar* menuBar = new wxMenuBar;
   
   m_MnuFile = new wxMenu;
   m_MnuFile->Append(ID_MNU_CONNECTING, _("&Connecting..."));
   m_MnuFile->Append(ID_MNU_DISCONNECTING, _("&Disconnecting"));
   m_MnuFile->AppendSeparator();
   m_MnuFile->Append(ID_MNU_QUIT, _("&Quit"));
   menuBar->Append(m_MnuFile, _("&File"));
   
   m_MnuLocal = new wxMenu;
   m_MnuLocal->Append(ID_MNU_UPLOAD, _("&Upload files"));
   m_MnuLocal->Append(ID_MNU_CHANGE_LOCAL, _("&Change directory..."));
   m_MnuLocal->Append(ID_MNU_CREATE_LOCAL, _("Create &directory..."));
   m_MnuLocal->Append(ID_MNU_DEL_LOCAL, _("&Erase files"));
   m_MnuLocal->Append(ID_MNU_REFRESH_LOCAL, _("&Refresh"));
   menuBar->Append(m_MnuLocal, _("Local"));
   
   m_MnuRemote = new wxMenu;
   m_MnuRemote->Append(ID_MNU_DOWNLOAD, _("&Download files"));
   m_MnuRemote->Append(ID_MNU_CREATE_REMOTE, _("&Create directory..."));
   m_MnuRemote->Append(ID_MNU_DEL_REMOTE, _("&Erase files"));
   m_MnuRemote->Append(ID_MNU_RENAME_REMOTE, _("Rename file..."));
   m_MnuRemote->Append(ID_MNU_REFRESH_REMOTE, _("&Refresh"));
   menuBar->Append(m_MnuRemote, _("Remote"));
   
   wxMenu* MnuHelp = new wxMenu;
   MnuHelp->Append(ID_MNU_ABOUT, _("&About..."));
   menuBar->Append(MnuHelp, _("&Help"));
   
   SetMenuBar(menuBar);

   wxStatusBar* StatusBar = new wxStatusBar(this, ID_STATUSBAR,
                                            wxST_SIZEGRIP|wxNO_BORDER);
   StatusBar->SetFieldsCount(3);
   SetStatusBar(StatusBar);
}

void wxMyFTPFrame::CreateToolbar()
{
   wxToolBar* ToolBar = CreateToolBar( wxTB_FLAT|wxTB_HORIZONTAL, ID_TOOLBAR );
   
   wxBitmap BmpConnecting(tool_connecting_xpm);
   ToolBar->AddTool(ID_MNU_CONNECTING, wxEmptyString, BmpConnecting, _("Connecting..."), wxITEM_NORMAL);
   
   wxBitmap BmpDisconnecting(tool_disconnecting_xpm);
   ToolBar->AddTool(ID_MNU_DISCONNECTING, wxEmptyString, BmpDisconnecting, _("Disconnecting"), wxITEM_NORMAL);
   
   ToolBar->AddSeparator();
   
   wxBitmap BmpUpload(tool_upload_xpm);
   ToolBar->AddTool(ID_MNU_UPLOAD, wxEmptyString, BmpUpload, _("Upload selected files"), wxITEM_NORMAL);
   
   wxBitmap BmpDownload(tool_download_xpm);
   ToolBar->AddTool(ID_MNU_DOWNLOAD, wxEmptyString, BmpDownload, _("Download selected files"), wxITEM_NORMAL);
   
   ToolBar->AddSeparator();
   
   wxBitmap BmpBinary(tool_binary_xpm);
   ToolBar->AddTool(ID_MNU_BINARY, wxEmptyString, BmpBinary, _("Binary mode"), wxITEM_RADIO);
   
   wxBitmap BmpAscii(tool_ascii_xpm);
   ToolBar->AddTool(ID_MNU_ASCII, wxEmptyString, BmpAscii, _("Ascii mode"), wxITEM_RADIO);
   
   wxBitmap BmpAuto(tool_auto_xpm);
   ToolBar->AddTool(ID_MNU_AUTO, wxEmptyString, BmpAuto, _("Auto mode"), wxITEM_RADIO);
   
   ToolBar->AddSeparator();
   
   ToolBar->Realize();
   SetToolBar(ToolBar);
   
   ToolBar->ToggleTool(ID_MNU_AUTO, true);
}

void wxMyFTPFrame::CreateControls()
{
   wxBoxSizer* Sizer1 = new wxBoxSizer(wxVERTICAL);
   SetSizer(Sizer1);

   m_TextCtrlInfo = new wxTextCtrl(this, ID_INFORMATION, wxEmptyString,
                                   wxDefaultPosition, wxSize(-1, 60),
                                   wxTE_MULTILINE|wxTE_READONLY);
   Sizer1->Add(m_TextCtrlInfo, 0, wxGROW, 0);

   m_HorzSplitter = new wxSplitterWindow(this, ID_SPLITTERWINDOW,
                                         wxDefaultPosition, wxDefaultSize,
                                         wxSP_3DBORDER|wxSP_3DSASH|wxNO_BORDER);
   Sizer1->Add(m_HorzSplitter, 1, wxGROW, 0);

   wxPanel* Panel1 = new wxPanel(m_HorzSplitter, ID_FILES_PANEL, wxDefaultPosition,
                                 wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL);
                                 
   m_TransfertList = new wxListCtrl(m_HorzSplitter, ID_TRANSFERTS_LIST,
                                    wxDefaultPosition, wxDefaultSize, wxLC_REPORT);

   m_HorzSplitter->SetSashGravity(1.0);
   m_HorzSplitter->SetMinimumPaneSize(60);
   m_HorzSplitter->SplitHorizontally(Panel1, m_TransfertList);

   wxBoxSizer* Sizer2 = new wxBoxSizer(wxHORIZONTAL);
   Panel1->SetSizer(Sizer2);

   m_LocalFiles = new wxListCtrl(Panel1, ID_LOCAL_FILES, wxDefaultPosition,
                                 wxDefaultSize, wxLC_REPORT|wxRAISED_BORDER|wxLC_VRULES);
   Sizer2->Add(m_LocalFiles, 1, wxGROW, 0);

   m_RemoteFiles = new wxListCtrl(Panel1, ID_REMOTE_FILES, wxDefaultPosition,
                                  wxDefaultSize, wxLC_REPORT|wxRAISED_BORDER|wxLC_VRULES);
   Sizer2->Add(m_RemoteFiles, 1, wxGROW, 0);

   m_TransfertList->InsertColumn(0, _("Job"), wxLIST_FORMAT_LEFT, 80);
   m_TransfertList->InsertColumn(1, _("File"), wxLIST_FORMAT_LEFT, 120);
   m_TransfertList->InsertColumn(2, _("Address"), wxLIST_FORMAT_LEFT, 120);
   m_TransfertList->InsertColumn(3, _("Bytes"), wxLIST_FORMAT_RIGHT, 90);
   m_TransfertList->InsertColumn(4, _("Status"), wxLIST_FORMAT_LEFT, 120);
   m_TransfertList->InsertColumn(5, _("%"), wxLIST_FORMAT_LEFT, 60);

   m_LocalFiles->InsertColumn(0, _("File"), wxLIST_FORMAT_LEFT, 150);
   m_LocalFiles->InsertColumn(1, _("Size"), wxLIST_FORMAT_RIGHT, 80);
   m_LocalFiles->InsertColumn(2, _("Date"), wxLIST_FORMAT_LEFT, 130);

   m_RemoteFiles->InsertColumn(0, _("File"), wxLIST_FORMAT_LEFT, 150);
   m_RemoteFiles->InsertColumn(1, _("Size"), wxLIST_FORMAT_RIGHT, 80);
   m_RemoteFiles->InsertColumn(2, _("Date"), wxLIST_FORMAT_LEFT, 130);

   wxImageList* imageList = wxTheFileIconsTable->GetSmallImageList();
   m_LocalFiles->SetImageList(imageList, wxIMAGE_LIST_SMALL);
   m_RemoteFiles->SetImageList(imageList, wxIMAGE_LIST_SMALL);

   UpdateDisplay();
}

void wxMyFTPFrame::OnMnuConnectingClick(wxCommandEvent& event)
{
   wxString sService;
   wxMyFTPCnxDlg cnxDlg(this);

   sService = wxString::Format(_T("%hu"), m_Service);
   cnxDlg.SetHostname(m_HostName);
   cnxDlg.SetService(sService);
   cnxDlg.SetUsername(m_User);
   cnxDlg.SetPassword(m_Password);

   if (cnxDlg.ShowModal() == wxID_OK)
   {
      m_HostName = cnxDlg.GetHostname();
      sService = cnxDlg.GetService();
      m_User = cnxDlg.GetUsername();
      m_Password = cnxDlg.GetPassword();

      unsigned long val;
      if (!sService.ToULong(&val)||(val < 1)||(val > 65535))
      {
         wxMessageBox(sService + _(" is a bad service number."), _("Error"));
         return;
      }
      m_Service = (unsigned short)val;
      
      wxMyFTP* MyFTP;
      wxManageRemoteThread* thread;

      // recherche ou cration du client ftp
      MyFTP = wxGetApp().CreateFTP(m_HostName, m_Service, m_User, m_Password, m_Passive);

      // Cration de l'instance du thread principal
      thread = new wxManageRemoteThread(this, MyFTP);
      if (thread)
      {  // Creation de la tche avec un peu de mmoire (4K)
         if (thread->Create(4096) != wxTHREAD_NO_ERROR)
         {
            m_TextCtrlInfo->AppendText(_("Can't create thread!\n"));
            thread->Delete();
         }
         else
         {  // Ajout de l'instance dans le tableau de tches
            wxCriticalSectionLocker enter(wxGetApp().m_array_thread_critsect);
            wxGetApp().m_threads.Add(thread);
            // Lancement de la tche
            if (thread->Run() != wxTHREAD_NO_ERROR)
            {
               m_TextCtrlInfo->AppendText(_("Can't start thread!\n"));
               thread->Delete();
            }
         }
      }
      else
      {  // L'instance n'a pas pu tre crer on supprime si possible
         // l'instance de wxMyFTP associe
         MyFTP->m_ftp_critsect.Enter();
         if (!MyFTP->OnStreaming() && !wxGetApp().Used(MyFTP))
         {
            wxCriticalSectionLocker enter(wxGetApp().m_array_ftp_critsect);
            wxGetApp().m_ftps.Remove(MyFTP);
            MyFTP->m_ftp_critsect.Leave();
            delete MyFTP;
         }
         else
            MyFTP->m_ftp_critsect.Leave();
      }
   }
   event.Skip();
}

void wxMyFTPFrame::OnMnuDisconnectingClick(wxCommandEvent& event)
{
   SetRemoteAction(mfjDisconnect);
   event.Skip();
}

void wxMyFTPFrame::OnMnuQuitClick(wxCommandEvent& event)
{
   Close();
   event.Skip();
}

void wxMyFTPFrame::OnMnuUploadClick(wxCommandEvent& event)
{
   PrepareTransfert(ttUpload);
   event.Skip();
}

void wxMyFTPFrame::OnMnuChangeLocalClick(wxCommandEvent& event)
{
   wxDirDialog DirDlg(this, _("Choose a directory"), wxGetCwd());

   if (DirDlg.ShowModal() == wxID_OK)
   {
      wxSetWorkingDirectory(DirDlg.GetPath());
      UpdateDisplay();
   }

   event.Skip();
}

void wxMyFTPFrame::OnMnuCreateLocalClick(wxCommandEvent& event)
{
   wxString dir, msg;

   dir = wxGetTextFromUser(_("Enter new directory name :"),
                           _("Create a new remote directory"), wxEmptyString, this);
   if (!dir.IsEmpty())
   {
      if (wxMkdir(dir))
      {
         msg = wxString::Format(_("Directory [%s] successfully created."), dir.c_str());
         UpdateDisplay();
      }
      else
         msg = wxString::Format(_("Unable to create directory [%s]."), dir.c_str());
      m_TextCtrlInfo->AppendText(msg);
      m_TextCtrlInfo->AppendText(_T("\n"));
   }
   event.Skip();
}

void wxMyFTPFrame::OnMnuDelLocalClick(wxCommandEvent& event)
{
   long item = -1;
   long data;
   wxString name, msg;
   int files, dirs;

   event.Skip();

   files = dirs = 0;
   do
   {
      item = m_LocalFiles->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

      if (item != -1)
      {
         data = m_LocalFiles->GetItemData(item);
         if (data < 0)
            ++dirs;
         else
            ++files;
      }
   }
   while (item != -1);

   if (files||dirs)
   {
      msg = _("Are you sure you want to delete ");
      if (files)
      {
         if (files > 1)
            msg += _("files ");
         else
            msg += _("file ");
      }
      if (files&&dirs)
         msg += _("and ");
      if (dirs)
      {
         if (dirs > 1)
            msg += _("directories ");
         else
            msg += _("directory ");
      }
      msg += ("?\n");
      if (wxMessageBox(msg, _("Question"), wxYES_NO)==wxNO)
         return;
   }

   do
   {
      item = m_LocalFiles->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

      if (item != -1)
      {
         name = m_LocalFiles->GetItemText(item);
         data = m_LocalFiles->GetItemData(item);
         if (data < 0)
         {
            if (wxRmdir(name))
               msg = wxString::Format(_("Directory [%s] successfully deleted."), name.c_str());
            else
               msg = wxString::Format(_("Unable to delete directory [%s]."), name.c_str());
         }
         else
         {
            if (wxRemoveFile(name))
               msg = wxString::Format(_("File [%s] successfully deleted."), name.c_str());
            else
               msg = wxString::Format(_("Unable to delete file [%s]."), name.c_str());
         }
         m_TextCtrlInfo->AppendText(msg);
         m_TextCtrlInfo->AppendText(_T("\n"));
      }
   }
   while (item != -1);

   UpdateDisplay();
}

void wxMyFTPFrame::OnMnuRefreshLocalClick(wxCommandEvent& event)
{
   UpdateDisplay();
   event.Skip();
}

void wxMyFTPFrame::OnMnuDownloadClick(wxCommandEvent& event)
{
   PrepareTransfert(ttDownload);
   event.Skip();
}

void wxMyFTPFrame::OnMnuCreateRemoteClick(wxCommandEvent& event)
{
   wxString tmp = wxGetTextFromUser(_("Enter new directory name :"),
                                    _("Create a new remote directory"),
                                    wxEmptyString, this);
   if (!tmp.IsEmpty())
      SetRemoteAction(mfjCreateDir, tmp);
   event.Skip();
}

void wxMyFTPFrame::OnMnuDelRemoteClick(wxCommandEvent& event)
{
   long item = -1;
   long data;
   wxString tmp;

   do
   {  // Rcuprer l'index du prochain item slectionn
      item = m_RemoteFiles->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

      if (item != -1)
      {  // Rcupration du nom et de la donne
         tmp = m_RemoteFiles->GetItemText(item);
         data = m_RemoteFiles->GetItemData(item);
         if (data < 0)  // Vrification si fichier ou rpertoire
            SetRemoteAction(mfjRemoveDir, tmp); //  en fonction du signe de la donne
         else
            SetRemoteAction(mfjDelete, tmp);
      }
   }
   while (item != -1);  // Boucle tant qu'un lment est trouv

   SetRemoteAction(mfjRefresh);  // Demande de raffraichissement de la liste
   event.Skip();
}

void wxMyFTPFrame::OnMnuRenameClick(wxCommandEvent& event)
{
   wxString tmp1, tmp2;
   long item = -1;
   // Rcuprer le premier lment slectionn
   item = m_RemoteFiles->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

   if (item != -1)
   {  // Rcupration du nom de l'lment et demande un autre nom  l'utilisateur
      tmp1 = m_RemoteFiles->GetItemText(item);
      tmp2 = wxGetTextFromUser(_("Enter new name :"),
                              _("Rename a remote file/directory"),
                              tmp1, this);
      if (tmp1 != tmp2)
      {  // Demande de changement de nom puis de raffraichissement
         SetRemoteAction(mfjRename, tmp1, tmp2);
         SetRemoteAction(mfjRefresh);
      }
   }
   event.Skip();
}

void wxMyFTPFrame::OnMnuRefreshRemoteClick(wxCommandEvent& event)
{  // Demande de raffraichissement
   SetRemoteAction(mfjRefresh);
   event.Skip();
}

void wxMyFTPFrame::OnMnuAboutClick(wxCommandEvent& event)
{
   wxString msg(_("MyFTP Version 0.1.0\n\n"
                  "A simple FTP client program.\n\n"
                  "Copyright (c) Fred Cailleau-Lepetit 2006\n"));

   wxMessageBox(msg, _T("About MyFTP"), wxOK | wxICON_INFORMATION, this);
   
   event.Skip();
}

void wxMyFTPFrame::OnMnuTransfertModeClick(wxCommandEvent& event)
{
   switch (event.GetInt())
   {
      case ID_MNU_BINARY :
         m_TransfertMode = TM_BINARY;
         break;
      case ID_MNU_ASCII  :
         m_TransfertMode = TM_ASCII;
         break;
      case ID_MNU_AUTO   :
         m_TransfertMode = TM_AUTO;
         break;
   }
   event.Skip();
}

void wxMyFTPFrame::OnEvtLaunchThread(wxCommandEvent& event)
{
   long lCount, i;
   wxTransJob* TransJob;
   wxMyFTP* MyFTP = 0;
   wxTransThread* TransThread;

   lCount = m_TransfertList->GetItemCount();
   for (i = 0; i < lCount; i++)  // Boucle sur les demande de transfert
   {  // Rcupration de la donne associe (wxTransJob*)
      TransJob = (wxTransJob*)m_TransfertList->GetItemData(i);

      if ((TransJob != 0)&&(TransJob->m_Thread == 0))
      {  // Si aucune instance de tche existe avec cet lment
         // Rcuprer ou crer une instance de wxMyFTP
         MyFTP = wxGetApp().CreateFTP(TransJob->m_Hostname, TransJob->m_Service,
                                      TransJob->m_User, TransJob->m_Password,
                                      TransJob->m_Passive);
         if (!MyFTP->OnStreaming())
            break;      // Aucun transfert en cours pour cet instance
         else
            MyFTP = 0;  // Un transfert est dj en cours on passe  l'lment suivant
      }
   }
   if (TransJob && MyFTP)  // Tout est prs pour tablir le transfert
   {  // Cration 'une instance de tche en fonstion du type de transfert
      if (TransJob->m_Type == ttUpload)   // Transfert montant
         TransThread = new wxUploadThread(this, MyFTP,
                                          TransJob->m_TransfertMode,
                                          TransJob->m_LocalPath,
                                          TransJob->m_RemotePath,
                                          TransJob->m_Filename);
      else  // Transfert descendant
         TransThread = new wxDownloadThread(this, MyFTP,
                                            TransJob->m_TransfertMode,
                                            TransJob->m_LocalPath,
                                            TransJob->m_RemotePath,
                                            TransJob->m_Filename);
      // On lance le thread de transfert
      if (TransThread)
      {  // Cration de la tche avec 4K de mmoire
         if (TransThread->Create(4096) != wxTHREAD_NO_ERROR)
         {
            m_TextCtrlInfo->AppendText(_("Can't create thread!\n"));
            TransThread->Delete();
         }
         else
         {  // Association de la tche dans les donnes de l'lement de liste
            TransJob->m_Thread = TransThread;
            // Ajout de la tche dans le tableau des tches
            wxCriticalSectionLocker enter(wxGetApp().m_array_thread_critsect);
            wxGetApp().m_threads.Add(TransThread);
            // Dmarrage de la tche
            if (TransThread->Run() != wxTHREAD_NO_ERROR)
            {
               TransJob->m_Thread = 0;
               m_TextCtrlInfo->AppendText(_("Can't start thread!\n"));
               TransThread->Delete();
            }
         }
      }
      else
      {  // La cration de l'instance de tche  choue
         // limination de l'instance de wxMyFTP si ncessaire
         MyFTP->m_ftp_critsect.Enter();
         if (!MyFTP->OnStreaming() && !wxGetApp().Used(MyFTP))
         {
            wxCriticalSectionLocker enter(wxGetApp().m_array_ftp_critsect);
            wxGetApp().m_ftps.Remove(MyFTP);
            MyFTP->m_ftp_critsect.Leave();
            delete MyFTP;
         }
         else
            MyFTP->m_ftp_critsect.Leave();
      }
   }
   event.Skip();
}

void wxMyFTPFrame::OnFilesChangeSelection(wxListEvent& event)
{
   EnableMenu();
   event.Skip();
}

void wxMyFTPFrame::OnLocalFilesBeginDrag(wxListEvent& event)
{
   event.Skip();
}

void wxMyFTPFrame::OnLocalFilesItemActivated(wxListEvent& event)
{
   wxListItem item = event.GetItem();
   wxString name = item.GetText();
   long l = item.GetData();

   if (l < 0)
   {  // C'est un rpertoire, changer le rpertoire courant
      wxSetWorkingDirectory(name);
      UpdateDisplay();
   }
   else  // C'est un fichier demande de transfert montant
      PrepareTransfert(ttUpload);
   event.Skip();
}

void wxMyFTPFrame::OnRemoteFilesBeginDrag(wxListEvent& event)
{
   event.Skip();
}

void wxMyFTPFrame::OnRemoteFilesItemActivated(wxListEvent& event)
{
   wxListItem item = event.GetItem();
   wxString name = item.GetText();  // Rcupration du nom
   long l = item.GetData();         // Rcupration de la donne associe

   if (l < 0)  // En fonction du signe de la donne
      SetRemoteAction(mfjChangeDir, name);   // Demande de Changement du rp. distant
   else
      PrepareTransfert(ttDownload);          // Demande de transfert descendant
   event.Skip();
}

void wxMyFTPFrame::UpdateDisplay()
{
   int image_id;
   long lCount;
   size_t count;
   wxArrayString files;
   wxString current, tmp;
   DIR *dp;
   dirent *ep;
   struct stat buf;

   current = wxGetCwd();
   m_LocalFiles->DeleteAllItems();

   dp = opendir(current.c_str());
   if (dp != NULL)
   {
      count = 0;
      while ((ep = readdir(dp)) != NULL)
      {
         tmp = ep->d_name;
         files.Add(tmp);
         count++; // le Data ne seras pas gal  zro
         if (tmp != _T("."))
         {
            if (stat(tmp.c_str(), &buf))
            {
               buf.st_mode  = 0;
               buf.st_size = 0;
               buf.st_atime = 0;
               buf.st_mtime = 0;
               buf.st_ctime = 0;
            }
            lCount = m_LocalFiles->GetItemCount();
            lCount = m_LocalFiles->InsertItem(lCount, tmp);

            if (S_ISDIR(buf.st_mode))
            {
               m_LocalFiles->SetItem(lCount, 1, _T("<DIR>"));
               m_LocalFiles->SetItemImage(lCount, wxFileIconsTable::folder);
               m_LocalFiles->SetItemData(lCount, -count);
            }
            else
            {
            #ifdef __WXMSW__
               m_LocalFiles->SetItem(lCount, 1, wxString::Format(_T("%lu"), buf.st_size));
            #else
               m_LocalFiles->SetItem(lCount, 1, wxString::Format(_T("%llu"), buf.st_size));
            #endif
               image_id = wxFileIconsTable::file;
               if (tmp.Find(_T('.')) != wxNOT_FOUND)
                   image_id = wxTheFileIconsTable->GetIconID(tmp.AfterLast(_T('.')));
               m_LocalFiles->SetItemImage(lCount, image_id);
               m_LocalFiles->SetItemData(lCount, count);
            }
            wxDateTime dateTime(buf.st_mtime);
            // The  ISO 8601 format is HH:MM:SS. I don't want the second
            m_LocalFiles->SetItem(lCount, 2, dateTime.FormatISODate() + _T(" ") + dateTime.FormatISOTime().Left(5));
         }
      }
      closedir (dp);
      m_LocalFiles->SortItems(CompareList, (long)&files);
   }
   EnableMenu();
}

void wxMyFTPFrame::EnableMenu()
{
   int countL = m_LocalFiles->GetSelectedItemCount();
   wxToolBar* toolBar = GetToolBar();
   if (wxGetApp().bConnected)
   {
      int countR = m_RemoteFiles->GetSelectedItemCount();
      m_MnuFile->Enable(ID_MNU_CONNECTING, false);
      m_MnuFile->Enable(ID_MNU_DISCONNECTING, true);
      m_MnuLocal->Enable(ID_MNU_UPLOAD, (countL > 0));
      m_MnuRemote->Enable(ID_MNU_DOWNLOAD, (countR > 0));
      m_MnuLocal->Enable(ID_MNU_DEL_LOCAL, (countL > 0));
      m_MnuRemote->Enable(ID_MNU_DEL_REMOTE, (countR > 0));
      m_MnuRemote->Enable(ID_MNU_RENAME_REMOTE, (countR == 1));
      m_MnuRemote->Enable(ID_MNU_CREATE_REMOTE, true);
      m_MnuRemote->Enable(ID_MNU_REFRESH_REMOTE, true);
      toolBar->EnableTool(ID_MNU_CONNECTING, false);
      toolBar->EnableTool(ID_MNU_DISCONNECTING, true);
      toolBar->EnableTool(ID_MNU_UPLOAD, (countL > 0));
      toolBar->EnableTool(ID_MNU_DOWNLOAD, (countR > 0));
   }
   else
   {
      m_MnuFile->Enable(ID_MNU_CONNECTING, true);
      m_MnuFile->Enable(ID_MNU_DISCONNECTING, false);
      toolBar->EnableTool(ID_MNU_CONNECTING, true);
      toolBar->EnableTool(ID_MNU_DISCONNECTING, false);
      m_MnuLocal->Enable(ID_MNU_DEL_LOCAL, (countL > 0));
      m_MnuLocal->Enable(ID_MNU_UPLOAD, false);
      m_MnuRemote->Enable(ID_MNU_DOWNLOAD, false);
      m_MnuRemote->Enable(ID_MNU_DEL_REMOTE, false);
      m_MnuRemote->Enable(ID_MNU_RENAME_REMOTE, false);
      m_MnuRemote->Enable(ID_MNU_CREATE_REMOTE, false);
      m_MnuRemote->Enable(ID_MNU_REFRESH_REMOTE, false);
      toolBar->EnableTool(ID_MNU_UPLOAD, false);
      toolBar->EnableTool(ID_MNU_DOWNLOAD, false);
   }
}

void wxMyFTPFrame::SetRemoteAction(wxFtpJobs action, const wxString& str1,
                                 const wxString& str2)
{
   wxJob* job;
   // Vrification qu'aucun transfert n'est en cours
   if (!wxGetApp().CurrentOnStreaming())
   {  // Blocage de l'accs exclusif au tableau de travaux
      wxCriticalSectionLocker enter(wxGetApp().m_array_job_critsect);

      job = new wxJob(action, str1, str2);   // Cration d'un instance de wxJob
      wxGetApp().m_jobs.Add(job);   // Ajout du travail dans la file d'attente
   }
   else
      m_TextCtrlInfo->AppendText(_("Can't execute command during transfer!\n"));
}

void wxMyFTPFrame::PrepareTransfert(wxTransfertType type)
{
   long item = -1;
   long data, fcount;
   wxListCtrl* list;
   wxString name, path;

   if (!wxGetApp().GetConnected())  // Vrification de la connexion
      return;

   fcount = 0;

   if (type == ttUpload)      // Transfert montant
   {
      path = wxGetCwd();      // Rcupration du rpertoire de dpart
      list = m_LocalFiles;    // Liste d'origine concerne par le transfert
   }
   else                       // Transfert descendant
   {
      path = wxGetApp().GetRemotePath();  // Rcupration du rpertoire de dpart
      list = m_RemoteFiles;   // Liste d'origine concerne par le transfert
   }

   item = list->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
   // Boucle tant d'un lment slectionn est trouv
   while (item != -1)
   {
      data = list->GetItemData(item);  // Rcupration de la donne associe
      if (data > 0) // donne positive c'est donc un fichier
      {
         name = list->GetItemText(item);

         if (type == ttUpload)
            Upload(path, name);
         else
            Download(path, name, m_HostName, m_Service, m_User,
                     m_Password, m_TransfertMode);
         ++fcount;
      }
      item = list->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
   }

   if (fcount)
   {  // Si des lments ont t ajouts, un message est envoy pour lancer le transfert
      wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, ID_CMD_LAUNCHTHREAD);
      wxPostEvent(this, event);
   }
}

void wxMyFTPFrame::Upload(const wxString& path, const wxString& name)
{
   long lCount;
   bool AsciiMode;
   wxTransJob* transjob;
   wxString remotePath;

   if (!wxGetApp().GetConnected())
      return;
   // Rcupre le chemin de destination
   remotePath = wxGetApp().GetRemotePath();

   if (m_TransfertMode == TM_ASCII)
      AsciiMode = true;             // Mode de transfert ASCII
   else if (m_TransfertMode == TM_BINARY)
      AsciiMode = false;            // Mode transfert Binaire
   else  // Sinon tabli le bon mode en fonction du type de fichier (extention)
      AsciiMode = IsAscii(name);    
   // cration d'une instance de wxTransJob
   transjob = new wxTransJob(remotePath, path, name, ttUpload,
                             m_HostName, m_Service, m_User, m_Password,
                             AsciiMode, m_Passive);
   // Ajout d'un lment dans la liste
   lCount = m_TransfertList->GetItemCount();
   lCount = m_TransfertList->InsertItem(lCount, _("Upload"));
   m_TransfertList->SetItem(lCount, 1, name);
   m_TransfertList->SetItem(lCount, 2, m_HostName);
   m_TransfertList->SetItem(lCount, 3, _T("0"));
   m_TransfertList->SetItem(lCount, 4, _("Waiting"));
   m_TransfertList->SetItem(lCount, 5, _T("0%"));
   // Association d'une instance de wxTransjob  l'lment
   m_TransfertList->SetItemData(lCount, (long)transjob);

}

void wxMyFTPFrame::Download(const wxString& path, const wxString& name,
                            const wxString& hostname, unsigned short service,
                            const wxString& user, const wxString& password,
                            int mode)
{
   bool AsciiMode;
   long lCount;
   wxTransJob* transjob;
   wxString localPath;

   localPath = wxGetCwd(); // Rcupration du chemin de destination

   if (mode == TM_ASCII)
      AsciiMode = true;          // Mode de transfert ASCII
   else if (mode == TM_BINARY)
      AsciiMode = false;         // Mode de transfert Binaire
   else  // Sinon tabli le mode de transfert en fonction du type de fichier
      AsciiMode = IsAscii(name);
   // Cration d'une instance de wTransJob
   transjob = new wxTransJob(path, localPath, name, ttDownload, hostname,
                             service, user, password, AsciiMode, m_Passive);
   // Insertion d'un lment dans la liste
   lCount = m_TransfertList->GetItemCount();
   lCount = m_TransfertList->InsertItem(lCount, _("Download"));
   m_TransfertList->SetItem(lCount, 1, name);
   m_TransfertList->SetItem(lCount, 2, hostname);
   m_TransfertList->SetItem(lCount, 3, _T("0"));
   m_TransfertList->SetItem(lCount, 4, _("Waiting"));
   m_TransfertList->SetItem(lCount, 5, _T("0%"));
   // Association de l'instance de wxTransJob  l'lment
   m_TransfertList->SetItemData(lCount, (long)transjob);

}

bool wxMyFTPFrame::IsAscii(const wxString& filename)
{
   wxString FileExt, Exts;

   Exts = ".TXT.HTM.HTML.XML.CSS.PHP.H.HPP.C.CPP.CC.";
   FileExt = _T(".") + wxFileName(filename).GetExt().Upper() + _T(".");

   return (Exts.Find(FileExt) != -1);
}
