/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : myftpframe.cpp                                                  */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 28/02/2006 */
/* email   : softinthebox@free.fr                      Rvision : 01/03/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/
#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "myftpframe.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include <wx/dirctrl.h>
#include "myftpframe.h"

#include "myftp_logo.xpm"
#include "tool_ascii.xpm"
#include "tool_auto.xpm"
#include "tool_binary.xpm"
#include "tool_connecting.xpm"
#include "tool_disconnecting.xpm"
#include "tool_download.xpm"
#include "tool_upload.xpm"

// Control Id
#define ID_TOOLBAR            10101
#define ID_STATUSBAR          10102
#define ID_INFORMATION        10103
#define ID_SPLITTERWINDOW     10104
#define ID_FILES_PANEL        10105
#define ID_LOCAL_FILES        10106
#define ID_REMOTE_FILES       10107
#define ID_TRANSFERTS_LIST    10108

// Menu Item Id
#define ID_MNU_CONNECTING     10201
#define ID_MNU_DISCONNECTING  10202
#define ID_MNU_QUIT           10203

#define ID_MNU_UPLOAD         10301
#define ID_MNU_CHANGE_LOCAL   10302
#define ID_MNU_CREATE_LOCAL   10303
#define ID_MNU_DEL_LOCAL      10304
#define ID_MNU_REFRESH_LOCAL  10305

#define ID_MNU_DOWNLOAD       10401
#define ID_MNU_CREATE_REMOTE  10402
#define ID_MNU_DEL_REMOTE     10403
#define ID_MNU_RENAME_REMOTE  10404
#define ID_MNU_REFRESH_REMOTE 10405

#define ID_MNU_ABOUT          10501

#define ID_MNU_BINARY         10601
#define ID_MNU_ASCII          10602
#define ID_MNU_AUTO           10603

wxMyFTPFrame::wxMyFTPFrame(wxWindow* parent, wxWindowID id, const wxString& caption,
                           const wxPoint& pos, const wxSize& size, long style)
{
   Create(parent, id, caption, pos, size, style);
}

wxMyFTPFrame::~wxMyFTPFrame()
{
}

bool wxMyFTPFrame::Create(wxWindow* parent, wxWindowID id, const wxString& caption,
                        const wxPoint& pos, const wxSize& size, long style)
{
   m_MnuFile = NULL;
   m_MnuLocal = NULL;
   m_MnuRemote = NULL;
   
   m_TextCtrlInfo = NULL;
   m_HorzSplitter = NULL;
   m_LocalFiles = NULL;
   m_RemoteFiles = NULL;
   m_TransfertList = NULL;

   wxFrame::Create(parent, id, caption, pos, size, style);
   CreateMenu();
   CreateToolbar();
   CreateControls();
   GetSizer()->Fit(this);
   GetSizer()->SetSizeHints(this);

   m_HorzSplitter->SetSashPosition(65);

   SetIcon(wxIcon(myftp_logo_xpm));
   return TRUE;
}

void wxMyFTPFrame::CreateMenu()
{
   wxMenuBar* menuBar = new wxMenuBar;
   
   m_MnuFile = new wxMenu;
   m_MnuFile->Append(ID_MNU_CONNECTING, _("&Connecting..."));
   m_MnuFile->Append(ID_MNU_DISCONNECTING, _("&Disconnecting"));
   m_MnuFile->AppendSeparator();
   m_MnuFile->Append(ID_MNU_QUIT, _("&Quit"));
   menuBar->Append(m_MnuFile, _("&File"));
   
   m_MnuLocal = new wxMenu;
   m_MnuLocal->Append(ID_MNU_UPLOAD, _("&Upload files"));
   m_MnuLocal->Append(ID_MNU_CHANGE_LOCAL, _("&Change directory..."));
   m_MnuLocal->Append(ID_MNU_CREATE_LOCAL, _("Create &directory..."));
   m_MnuLocal->Append(ID_MNU_DEL_LOCAL, _("&Erase files"));
   m_MnuLocal->Append(ID_MNU_REFRESH_LOCAL, _("&Refresh"));
   menuBar->Append(m_MnuLocal, _("Local"));
   
   m_MnuRemote = new wxMenu;
   m_MnuRemote->Append(ID_MNU_DOWNLOAD, _("&Download files"));
   m_MnuRemote->Append(ID_MNU_CREATE_REMOTE, _("&Create directory..."));
   m_MnuRemote->Append(ID_MNU_DEL_REMOTE, _("&Erase files"));
   m_MnuRemote->Append(ID_MNU_RENAME_REMOTE, _("Rename file..."));
   m_MnuRemote->Append(ID_MNU_REFRESH_REMOTE, _("&Refresh"));
   menuBar->Append(m_MnuRemote, _("Remote"));
   
   wxMenu* MnuHelp = new wxMenu;
   MnuHelp->Append(ID_MNU_ABOUT, _("&About..."));
   menuBar->Append(MnuHelp, _("&Help"));
   
   SetMenuBar(menuBar);

   wxStatusBar* StatusBar = new wxStatusBar(this, ID_STATUSBAR,
                                            wxST_SIZEGRIP|wxNO_BORDER);
   StatusBar->SetFieldsCount(3);
   SetStatusBar(StatusBar);
}

void wxMyFTPFrame::CreateToolbar()
{
   wxToolBar* ToolBar = CreateToolBar( wxTB_FLAT|wxTB_HORIZONTAL, ID_TOOLBAR );
   
   wxBitmap BmpConnecting(tool_connecting_xpm);
   ToolBar->AddTool(ID_MNU_CONNECTING, wxEmptyString, BmpConnecting, _("Connecting..."), wxITEM_NORMAL);
   
   wxBitmap BmpDisconnecting(tool_disconnecting_xpm);
   ToolBar->AddTool(ID_MNU_DISCONNECTING, wxEmptyString, BmpDisconnecting, _("Disconnecting"), wxITEM_NORMAL);
   
   ToolBar->AddSeparator();
   
   wxBitmap BmpUpload(tool_upload_xpm);
   ToolBar->AddTool(ID_MNU_UPLOAD, wxEmptyString, BmpUpload, _("Upload selected files"), wxITEM_NORMAL);
   
   wxBitmap BmpDownload(tool_download_xpm);
   ToolBar->AddTool(ID_MNU_DOWNLOAD, wxEmptyString, BmpDownload, _("Download selected files"), wxITEM_NORMAL);
   
   ToolBar->AddSeparator();
   
   wxBitmap BmpBinary(tool_binary_xpm);
   ToolBar->AddTool(ID_MNU_BINARY, wxEmptyString, BmpBinary, _("Binary mode"), wxITEM_RADIO);
   
   wxBitmap BmpAscii(tool_ascii_xpm);
   ToolBar->AddTool(ID_MNU_ASCII, wxEmptyString, BmpAscii, _("Ascii mode"), wxITEM_RADIO);
   
   wxBitmap BmpAuto(tool_auto_xpm);
   ToolBar->AddTool(ID_MNU_AUTO, wxEmptyString, BmpAuto, _("Auto mode"), wxITEM_RADIO);
   
   ToolBar->AddSeparator();
   
   ToolBar->Realize();
   SetToolBar(ToolBar);
   
   ToolBar->ToggleTool(ID_MNU_AUTO, true);
}

void wxMyFTPFrame::CreateControls()
{
   wxBoxSizer* Sizer1 = new wxBoxSizer(wxVERTICAL);
   SetSizer(Sizer1);

   m_TextCtrlInfo = new wxTextCtrl(this, ID_INFORMATION, wxEmptyString,
                                   wxDefaultPosition, wxSize(-1, 60),
                                   wxTE_MULTILINE|wxTE_READONLY);
   Sizer1->Add(m_TextCtrlInfo, 0, wxGROW, 0);

   m_HorzSplitter = new wxSplitterWindow(this, ID_SPLITTERWINDOW,
                                         wxDefaultPosition, wxDefaultSize,
                                         wxSP_3DBORDER|wxSP_3DSASH|wxNO_BORDER);
   Sizer1->Add(m_HorzSplitter, 1, wxGROW, 0);

   wxPanel* Panel1 = new wxPanel(m_HorzSplitter, ID_FILES_PANEL, wxDefaultPosition,
                                 wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL);
                                 
   m_TransfertList = new wxListCtrl(m_HorzSplitter, ID_TRANSFERTS_LIST,
                                    wxDefaultPosition, wxDefaultSize, wxLC_REPORT);

   m_HorzSplitter->SetSashGravity(1.0);
   m_HorzSplitter->SetMinimumPaneSize(60);
   m_HorzSplitter->SplitHorizontally(Panel1, m_TransfertList);

   wxBoxSizer* Sizer2 = new wxBoxSizer(wxHORIZONTAL);
   Panel1->SetSizer(Sizer2);

   m_LocalFiles = new wxListCtrl(Panel1, ID_LOCAL_FILES, wxDefaultPosition,
                                 wxDefaultSize, wxLC_REPORT|wxRAISED_BORDER|wxLC_VRULES);
   Sizer2->Add(m_LocalFiles, 1, wxGROW, 0);

   m_RemoteFiles = new wxListCtrl(Panel1, ID_REMOTE_FILES, wxDefaultPosition,
                                  wxDefaultSize, wxLC_REPORT|wxRAISED_BORDER|wxLC_VRULES);
   Sizer2->Add(m_RemoteFiles, 1, wxGROW, 0);

   m_TransfertList->InsertColumn(0, _("Job"), wxLIST_FORMAT_LEFT, 80);
   m_TransfertList->InsertColumn(1, _("File"), wxLIST_FORMAT_LEFT, 120);
   m_TransfertList->InsertColumn(2, _("Address"), wxLIST_FORMAT_LEFT, 120);
   m_TransfertList->InsertColumn(3, _("Bytes"), wxLIST_FORMAT_RIGHT, 90);
   m_TransfertList->InsertColumn(4, _("Status"), wxLIST_FORMAT_LEFT, 120);
   m_TransfertList->InsertColumn(5, _("%"), wxLIST_FORMAT_LEFT, 60);

   m_LocalFiles->InsertColumn(0, _("File"), wxLIST_FORMAT_LEFT, 150);
   m_LocalFiles->InsertColumn(1, _("Size"), wxLIST_FORMAT_RIGHT, 80);
   m_LocalFiles->InsertColumn(2, _("Date"), wxLIST_FORMAT_LEFT, 130);

   m_RemoteFiles->InsertColumn(0, _("File"), wxLIST_FORMAT_LEFT, 150);
   m_RemoteFiles->InsertColumn(1, _("Size"), wxLIST_FORMAT_RIGHT, 80);
   m_RemoteFiles->InsertColumn(2, _("Date"), wxLIST_FORMAT_LEFT, 130);

   wxImageList* imageList = wxTheFileIconsTable->GetSmallImageList();
   m_LocalFiles->SetImageList(imageList, wxIMAGE_LIST_SMALL);
   m_RemoteFiles->SetImageList(imageList, wxIMAGE_LIST_SMALL);

}
