/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : myftpframe.h                                                    */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 28/02/2006 */
/* email   : softinthebox@free.fr                      Rvision : 05/04/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2).

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/
#ifndef _MYFTPFRAME_H_
#define _MYFTPFRAME_H_

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "myftpframe.cpp"
#endif
#include <wx/splitter.h>
#include <wx/listctrl.h>
#include "myftpapp.h"

// Transfert Mode
#define TM_ASCII  0
#define TM_BINARY 1
#define TM_AUTO   2

#define ID_CMD_LAUNCHTHREAD   10701

class wxTransThread;

enum wxTransfertType {ttUpload, ttDownload};

class wxTransJob
{
   public:

      wxTransJob(const wxString& remote, const wxString& local,
                 const wxString& file, wxTransfertType type,
                 const wxString& hostname, unsigned short service,
                 const wxString& user, const wxString& password,
                 int mode, bool passive)
         {
            m_RemotePath = remote;
            m_LocalPath = local;
            m_Filename = file;
            m_Type = type;
            m_Hostname = hostname;
            m_Service = service;
            m_User = user;
            m_Password = password;
            m_TransfertMode = mode;
            m_Passive = passive;
            m_Thread = 0;
         }

      wxString m_RemotePath;
      wxString m_LocalPath;
      wxString m_Filename;
      wxTransfertType m_Type;

      wxString m_Hostname;
      unsigned short m_Service;
      wxString m_User;
      wxString m_Password;
      int m_TransfertMode;
      bool m_Passive;

      wxTransThread* m_Thread;
};

class wxMyFTPFrame: public wxFrame
{
   DECLARE_EVENT_TABLE()

   public:

      wxMyFTPFrame(wxWindow* parent, wxWindowID id = -1,
                 const wxString& caption = _("myFTP v0.1"),
                 const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxDefaultSize,
                 long style = wxDEFAULT_FRAME_STYLE);

      ~wxMyFTPFrame();

      bool Create(wxWindow* parent, wxWindowID id = -1,
                  const wxString& caption = _("myFTP v0.1"),
                  const wxPoint& pos = wxDefaultPosition,
                  const wxSize& size = wxDefaultSize,
                  long style = wxDEFAULT_FRAME_STYLE);

      void EnableMenu();

      void Upload(const wxString& path, const wxString& name);
      void Download(const wxString& path, const wxString& name,
                    const wxString& hostname, unsigned short service,
                    const wxString& user, const wxString& password, int mode);

      wxTextCtrl* GetTextCtrlInfo(){return m_TextCtrlInfo;}
      wxListCtrl* GetTransfertList(){return m_TransfertList;}
      wxListCtrl* GetLocalFiles(){return m_LocalFiles;}
      wxListCtrl* GetRemoteFiles(){return m_RemoteFiles;}

   protected:

      void CreateMenu();
      void CreateToolbar();
      void CreateControls();

      void OnMnuConnectingClick(wxCommandEvent& event);
      void OnMnuDisconnectingClick(wxCommandEvent& event);
      void OnMnuQuitClick(wxCommandEvent& event);
      void OnMnuUploadClick(wxCommandEvent& event);
      void OnMnuChangeLocalClick(wxCommandEvent& event);
      void OnMnuCreateLocalClick(wxCommandEvent& event);
      void OnMnuDelLocalClick(wxCommandEvent& event);
      void OnMnuRefreshLocalClick(wxCommandEvent& event);
      void OnMnuDownloadClick(wxCommandEvent& event);
      void OnMnuCreateRemoteClick(wxCommandEvent& event);
      void OnMnuDelRemoteClick(wxCommandEvent& event);
      void OnMnuRenameClick(wxCommandEvent& event);
      void OnMnuRefreshRemoteClick(wxCommandEvent& event);
      void OnMnuAboutClick(wxCommandEvent& event);
      void OnMnuTransfertModeClick(wxCommandEvent& event);

      void OnEvtLaunchThread(wxCommandEvent& event);
      
      void OnLocalFilesBeginDrag(wxListEvent& event);
      void OnLocalFilesItemActivated(wxListEvent& event);
      void OnRemoteFilesBeginDrag(wxListEvent& event);
      void OnRemoteFilesItemActivated(wxListEvent& event);
      void OnFilesChangeSelection(wxListEvent& event);

   private:

      wxString m_User;
      wxString m_Password;
      wxString m_HostName;
      unsigned short m_Service;
      int m_TransfertMode;
      bool m_Passive;

      wxMenu* m_MnuFile;
      wxMenu* m_MnuLocal;
      wxMenu* m_MnuRemote;

      wxTextCtrl* m_TextCtrlInfo;
      wxSplitterWindow* m_HorzSplitter;
      wxListCtrl* m_LocalFiles;
      wxListCtrl* m_RemoteFiles;
      wxListCtrl* m_TransfertList;

      void UpdateDisplay();

      void SetRemoteAction(wxFtpJobs action,
                           const wxString& str1 = wxEmptyString,
                           const wxString& str2 = wxEmptyString);
      void PrepareTransfert(wxTransfertType type);
      bool IsAscii(const wxString& filename);

};
#endif   // _MYFTPFRAME_H_
