/*---------------------------------------------------------------------------*/
/* Logiciel Client FTP.                                                      */
/*---------------------------------------------------------------------------*/
/* Projet  : MyFTP                                     Version  : 0.1.0.0    */
/* Fichier : transthread.cpp                                                  */
/* Auteur  : Fred Cailleau-Lepetit                     Date     : 03/04/2006 */
/* email   : softinthebox@free.fr                      Rvision : 03/04/2006 */
/*---------------------------------------------------------------------------*/
/* Copyright (C) Fred Cailleau-Lepetit 2005-2006                             */
/* Licence GNU General Public License  http://www.fsf.org/copyleft/gpl.html  */
/*---------------------------------------------------------------------------*/
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation (version 2);

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
selon les termes de la Licence Publique Gnrale GNU publie par la
Free Software Foundation (version 2).

Ce programme est distribu car potentiellement utile, mais
SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris
les garanties de commercialisation ou d'adaptation dans un but
spcifique. Reportez-vous  la Licence Publique Gnrale GNU
pour plus de dtails.
*/

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "transthread.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
   #include <wx/wx.h>
#endif

#include "transthread.h"
#include "myftpframe.h"

wxTransThread::wxTransThread(wxMyFTPFrame* frame, wxMyFTP* ftp, bool asciimode,
                             const wxString& localpath, const wxString& remotepath,
                             const wxString& filename) :
               wxBaseThread(frame, ftp)
{
   m_AsciiMode = asciimode;
   m_LocalPath = localpath;
   m_RemotePath = remotepath;
   m_FileName = filename;
   m_FileSize = m_SizeRead = 0;
}

wxTransThread::~wxTransThread()
{
   // Avant de sortir envoyer un message de lancement de nouveau transfert
   wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, ID_CMD_LAUNCHTHREAD);
   wxPostEvent(m_pFrame, event);
}

void wxTransThread::UpdateTansfertItemStatus(wxUpdateMode mode)
{
   if (!TestDestroy())        // Si aucune demande de destruction
   {
      long lCount;
      wxString sStatus;

      wxMutexGuiEnter();      // Utilisation exclusive de la GUI

      lCount = GetTransfertItemIndex();   // Rcupre l'index
      if (lCount != -1)
      {  // Prparation de la chaine de status
         switch (mode)
         {
            case umStarting :
               sStatus = _("Starting");
               break;
            case umWaiting :
               sStatus = _("Waiting");
               break;
            case umTransfering :
               sStatus = _("Transfering");
               break;
            default :
               sStatus = _("Finishing");
         }
         // Affichage de la chaine
         m_pFrame->GetTransfertList()->SetItem(lCount, 4, sStatus);
      }

      wxMutexGuiLeave();      // Fin de l'utilisation exclusive de la GUI
   }
}

void wxTransThread::UpdateTransfertItem()
{
   if (!TestDestroy())        // Si aucune demande de destruction
   {
      long lCount;
      wxString sNbBytes, sPCent;

      wxMutexGuiEnter();      // Utilisation exclusive de la GUI

      lCount = GetTransfertItemIndex();   // Rcupre l'index
      if (lCount != -1)
      {
         // Affiche le nombre d'octets lus
         sNbBytes = wxString::Format(_T("%u"), m_SizeRead);
         m_pFrame->GetTransfertList()->SetItem(lCount, 3, sNbBytes);
         // Si une taille de fichier  t trouv
         if (m_FileSize != 0)
         {  // Calculer le pourcentage lu et l'afficher
            double dPCent = (double)m_SizeRead * 100.0 / (double)m_FileSize;
            if (dPCent > 100.0)
               dPCent = 100.0;
            sPCent = wxString::Format(_T("%u%%"), (size_t)dPCent);
         }
         else  // Taille non trouve
            sPCent = _T("N/A");
         m_pFrame->GetTransfertList()->SetItem(lCount, 5, sPCent);
      }

      wxMutexGuiLeave();      // Fin de l'utilisation exclusive de la GUI
   }
}

void wxTransThread::DeleteTransfertItem()
{
   if (!TestDestroy())        // Si aucune demande de destruction
   {
      long lCount;

      wxMutexGuiEnter();      // Utilisation exclusive de la GUI

      lCount = GetTransfertItemIndex();   // Rcupre l'index
      if (lCount != -1) // Supprime l'item  l'index trouv
         m_pFrame->GetTransfertList()->DeleteItem(lCount);

      wxMutexGuiLeave();      // Fin de l'utilisation exclusive de la GUI
   }
}

long wxTransThread::GetTransfertItemIndex()
{
   long lCount, i;
   wxTransJob* TransJob;

   lCount = m_pFrame->GetTransfertList()->GetItemCount();
   // Rcupre l'index de l'item qui  son data (wxTransJob*) dont la donne m_Thread
   // est identique au pointeur de la tche
   for (i = 0; i < lCount; i++)
   {
      TransJob = (wxTransJob*)m_pFrame->GetTransfertList()->GetItemData(i);
      if ((TransJob != 0)&&(TransJob->m_Thread == this))
         return i;
   }
   return -1;
}
